/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2009 C ESCOBAR - christel.escobar@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.controls.pesv2.sig;

import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.xemelios.controls.AbstractUnitControl;
import fr.gouv.finances.cp.xemelios.controls.Anomalie;
import fr.gouv.finances.cp.xemelios.controls.IdGenerator;
import fr.gouv.finances.cp.xemelios.controls.Node;
import fr.gouv.finances.cp.xemelios.controls.pesv2.ConstantsControlsPESv2;
import fr.gouv.finances.dgfip.utils.xml.certs.Certificate509;

/**
 * Implmente le contrle N SIG3
 * 
 * Il consiste  vrifier la rgle fonctionnelle suivante :
 * 
 * Les lments de la signature lectronique suivants sont obligatoires en cas de signature lectronique (niveau global ou bordereau) et doivent tre valoriss correctement :
 * - Nom du signataire
 * - Qualit (Rle) du signataire
 * - Date de la signature
 * - Cumul des montants annuels
 * (Svrit Bloquant)
 * 
 * Les lments de la signature lectronique suivants sont souhaitables en cas de signature lectronique (niveau global ou bordereau) et doivent tre valoriss correctement :
 * - Adresse du signataire
 * (Svrit Non Bloquant)
 * 		
 * 
 * 
 * Il est fait ainsi :
 * 
 * Si prsence dune balise <ds :Signature> (niveau global ou bordereau), vrifier la prsence des lments obligatoires de signature lectronique suivants :
 *	-	<Nom du signataire> (encrypt dans ds:Signature/ds:KeyInfo/ds:X509Data/ds:X509Certificate)
 *	-	/ds:Signature/ds:Object/xad:QualifyingProperties/xad:SignedProperties/xad:SignedSignatureProperties/xad:SignerRole/xad:ClaimedRoles/xad:ClaimedRole
 *	-	/ds:Signature/ds:Object/xad:QualifyingProperties/xad:SignedProperties /xad:SigningTime>
 *	-	< Bordereau /BlocBordereau/MtCumulAnnuel >
 *	(Svrit Bloquante)
 *
 * Si prsence dune balise <ds :Signature> (niveau global ou bordereau), vrifier la prsence des lments facultatifs suivants : 
 *	-	Adresse signature ds:Signature/ds:Object/xad:QualifyingProperties/xad:SignedProperties/xad:SignedSignatureProperties/xad:SignatureProductionPlace
 *	(Svrit Non Bloquant)
 * 		
 */

public class SIG3 extends AbstractUnitControl implements ConstantsControlsPESv2 {
	private final static Logger logger = Logger.getLogger(SIG3.class);
    public static final transient String CTRL_ID = "SIG3";
	private Hashtable<String,Object> hParams;
	
	/**
	 * La balise du document gnral
	 */
	private String docId = null;
	private String bord_nodeId = null;

	/**
	 * Variables devant tre remplaces dans le message
	 */
	private static final String MSG_ERREUR = "#MSG_ERREUR#";
	
	/**
	 * Variables contenant les valeurs ncessaires  ce contrle
	 */

	private String bord_Exer = "";
	private String bord_IdBord = "";
	private String bord_TypBord = "";
	private String piece_IdPce = "";
	
	private String sign_nom = "";
	private String sign_qualite = "";
	private String sign_date = "";
	private String bord_mtCumul = "";
	private Vector<String> montantsCumul = new Vector<String>();
	private String sign_adresse = "";
	private boolean sign_isPresent = false;
	private boolean sign_nom_isPresent = false;
	private boolean sign_date_isPresent = false;
	private boolean sign_adresse_isPresent = false;
	private boolean sign_qualite_isPresent = false;
	
	StringBuffer nom;
	StringBuffer qualite;
	StringBuffer date;
	StringBuffer adresse;
	
	/**
	 * Chemins incomplet vers les divers lments que l'on veux lire. Ils sont incomplets dans la mesure 
	 * o on n'a pas le dbut du chemin, vu que ce contrle est utilis pour les Dpenses et les Recettes
	 */
	private static final String CHEMIN_BLOCBORDEREAU = "/Bordereau/BlocBordereau/";
	private static final String CHEMIN_BLOCBORDEREAU_EXER = "/Bordereau/BlocBordereau/Exer/";
	private static final String CHEMIN_BLOCBORDEREAU_IDBORD = "/Bordereau/BlocBordereau/IdBord/";
	private static final String CHEMIN_BLOCBORDEREAU_TYPBORD = "/Bordereau/BlocBordereau/TypBord/";
	private static final String CHEMIN_BLOCBORDEREAU_MTCUMUL = "/Bordereau/BlocBordereau/MtCumulAnnuel/";
	private static final String CHEMIN_SIGNATURE = "/Signature/";
	private static final String CHEMIN_SIGNATURE_NOM = "/Signature/KeyInfo/X509Data/X509Certificate/";
	private static final String CHEMIN_SIGNATURE_QUALITE = "/Signature/Object/QualifyingProperties/SignedProperties/SignedSignatureProperties/SignerRole/ClaimedRoles/ClaimedRole/";
	private static final String CHEMIN_SIGNATURE_DATE = "/Signature/Object/QualifyingProperties/SignedProperties/SignedSignatureProperties/SigningTime/";
	private static final String CHEMIN_SIGNATURE_ADRESSE = "/Signature/Object/QualifyingProperties/SignedProperties/SignedSignatureProperties/SignatureProductionPlace/City/";
	
	private static final String CHEMIN_SIGNATURE_BORD = "/Bordereau/Signature/";
	private static final String CHEMIN_SIGNATURE_GLOBALE = "PES_Aller/Signature/";
	
	/**
	 * Selon que l'on est en RECETTE ou DEPENSE, le xpath est diffrent
	 */
	private static final String RECETTE = "PES_RecetteAller";
	private static final String DEPENSE = "PES_DepenseAller";
	private boolean depense = false;

	private static String CHEMIN_BLOCPIECE = "";
	private static String CHEMIN_BLOCPIECE_IDPCE = "";
	private static final String R_CHEMIN_BLOCPIECE = "/Bordereau/Piece/BlocPiece/";
	private static final String R_CHEMIN_BLOCPIECE_IDPCE = "/Bordereau/Piece/BlocPiece/IdPce/";
	private static final String D_CHEMIN_BLOCPIECE = "/Bordereau/Piece/BlocPiece/";
	private static final String D_CHEMIN_BLOCPIECE_IDPCE = "/Bordereau/Piece/BlocPiece/InfoPce/IdPce/";

	
		
	
	public void startDocument() throws SAXException {
		logger.info("In "+CTRL_ID);
	}
	
	public void setDocId(String docID) {
		this.docId = docID;		
	}

	public void startElement(String uri, String localName, String qName,Attributes atts, String xpath) {
		
		// RECETTE ou DEPENSE
		if (xpath.indexOf(DEPENSE)!=-1) {
			depense = true;
			CHEMIN_BLOCPIECE = D_CHEMIN_BLOCPIECE;
			CHEMIN_BLOCPIECE_IDPCE = D_CHEMIN_BLOCPIECE_IDPCE;

		}
		if (xpath.indexOf(RECETTE)!=-1) {
			depense = false;
			CHEMIN_BLOCPIECE = R_CHEMIN_BLOCPIECE;
			CHEMIN_BLOCPIECE_IDPCE = R_CHEMIN_BLOCPIECE_IDPCE;

		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU)) {
			bord_Exer = bord_IdBord = bord_TypBord = bord_mtCumul = "";
			bord_nodeId = atts.getValue("ano:node-id");
		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU_EXER)) {
			bord_Exer = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU_IDBORD)) {
			bord_IdBord = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU_TYPBORD)) {
			bord_TypBord = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU_MTCUMUL)) {
			bord_mtCumul = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_BLOCPIECE)) {
			piece_IdPce = "";
		}
		if (xpath.endsWith(CHEMIN_BLOCPIECE_IDPCE)) {
			piece_IdPce = atts.getValue("V");
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE)) {
			sign_isPresent = true;
			sign_adresse = sign_date = sign_nom = sign_qualite = "";
			nom = new StringBuffer();
			qualite = new StringBuffer();
			date = new StringBuffer();
			adresse = new StringBuffer();
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_NOM)) {
			sign_nom_isPresent = true;
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_QUALITE)) {
			sign_qualite_isPresent = true;
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_DATE)) {
			sign_date_isPresent = true;
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_ADRESSE)) {
			sign_adresse_isPresent = true;
		}

	}
	
	public Vector<Anomalie> endElement(String uri, String localName, String qName, String xpath) throws SAXException {

		String erreur = "";
		Vector<Anomalie> ano1 = new Vector<Anomalie>();
		String message = "";
		
		if (xpath.endsWith(CHEMIN_BLOCBORDEREAU)) {
			montantsCumul.add(bord_mtCumul);
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_NOM)) {
			sign_nom_isPresent = false;
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_QUALITE)) {
			sign_qualite_isPresent = false;
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_DATE)) {
			sign_date_isPresent = false;
		}
		if (xpath.endsWith(CHEMIN_SIGNATURE_ADRESSE)) {
			sign_adresse_isPresent = false;
		}
		
		
		// signature niveau bordereau
		if (sign_isPresent && xpath.endsWith(CHEMIN_SIGNATURE_BORD)){
			String nomSansEspaces = nom.toString().replace(" ", "").replace("\n", "").replace("\t", "");
			
			if (nomSansEspaces.length()>0)
				sign_nom = Certificate509.extractCN(nomSansEspaces);
			if (qualite.length()>0)
				sign_qualite = qualite.toString();
			if (date.length()>0)
				sign_date = date.toString();
			if (adresse.length()>0)
				sign_adresse = adresse.toString();

			
			if (sign_nom.equals("") || sign_nom.equals(" ") || sign_nom.equals("0")
					|| sign_qualite.equals("") || sign_qualite.equals(" ") || sign_qualite.equals("0")
					|| sign_date.equals("") || sign_date.equals(" ") || sign_date.equals("0")
					|| bord_mtCumul.equals("") || bord_mtCumul.equals(" ") || bord_mtCumul.equals("0")){
				
				erreur = "Absence des lments obligatoires relatifs  la signature lectronique : Nom du signataire, Qualit du signataire, date de la signature, cumul des montants annuels (BLOQUANT)";
				
				String msg = getDocumentModel().getControlById(CTRL_ID).getMessage().getHtmlContent();
				msg = msg.replaceAll(MSG_ERREUR, erreur);
				message = "Bordereau n " + bord_IdBord;

				
				String path="@added:primary-key='"+bord_Exer+"-"+bord_TypBord+"-"+bord_IdBord+"'";
				String anoId = IdGenerator.nextId();
				String libelleLien = "Bordereau "+bord_IdBord+" Mandat "+piece_IdPce;
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Bordereau");
				xslParams.put("mandatId", piece_IdPce);
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
                                        getDocumentModel().getControlById(CTRL_ID).getId(),
                                        getDocumentModel().getControlById(CTRL_ID).getLibelle(),
                                        getDocumentModel().getDocumentId(),
                                        xpath.split("/")[1],
                                        (depense?BORDEREAU_DEPENSE:BORDEREAU_RECETTE), // lment importable
                                        bord_IdBord, // id lment importable
                                        localName,
                                        libelleLien,
                                        message + " - " + msg,
                                        getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
                                        getDocumentModel().getControlById(CTRL_ID).getSeverity(),
                                        path,
                                        xslParams);
				ano.addNode(new Node(bord_nodeId));
				ano1.add(ano);
			
			}
			
			if (sign_adresse.equals("") || sign_adresse.equals(" ") || sign_adresse.equals("0")){
				
				erreur = "Absence de l'lment relatif suivant : Lieu de la signature lectronique (NON BLOQUANT)";
				
				String msg = getDocumentModel().getControlById(CTRL_ID).getMessage().getHtmlContent();
				msg = msg.replaceAll(MSG_ERREUR, erreur);
				message = "Bordereau n " + bord_IdBord;

				
				String path="@added:primary-key='"+bord_Exer+"-"+bord_TypBord+"-"+bord_IdBord+"'";
				String anoId = IdGenerator.nextId();
				String libelleLien = "Bordereau "+bord_IdBord+" Mandat "+piece_IdPce;
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Bordereau");
				xslParams.put("mandatId", piece_IdPce);
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
                                        getDocumentModel().getControlById(CTRL_ID).getId(),
                                        getDocumentModel().getControlById(CTRL_ID).getLibelle(),
                                        getDocumentModel().getDocumentId(),
                                        xpath.split("/")[1],
                                        (depense?BORDEREAU_DEPENSE:BORDEREAU_RECETTE), // lment importable
                                        bord_IdBord, // id lment importable
                                        localName,
                                        libelleLien,
                                        message + " - " + msg,
                                        getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
                                        "NON BLOQUANT",
                                        path,
                                        xslParams);
				ano.addNode(new Node(bord_nodeId));
				ano1.add(ano);
			}
			return ano1;
			
		}
		
		
		// signature globale
		if (sign_isPresent && xpath.endsWith(CHEMIN_SIGNATURE_GLOBALE)){
			String nomSansEspaces = nom.toString().replace(" ", "").replace("\n", "").replace("\t", "");
			boolean err = false;
			
			if (nomSansEspaces.length()>0)
				sign_nom = Certificate509.extractCN(nomSansEspaces);
			if (qualite.length()>0)
				sign_qualite = qualite.toString();
			if (date.length()>0)
				sign_date = date.toString();
			if (adresse.length()>0)
				sign_adresse = adresse.toString();

			
			for (int i = 0; i < montantsCumul.size(); i++){
				if (montantsCumul.get(i).equals("") || montantsCumul.get(i).equals(" ") || montantsCumul.get(i).equals("0"))
					err = true;
			}
			
			
			if (sign_nom.equals("") || sign_nom.equals(" ") || sign_nom.equals("0")
					|| sign_qualite.equals("") || sign_qualite.equals(" ") || sign_qualite.equals("0")
					|| sign_date.equals("") || sign_date.equals(" ") || sign_date.equals("0") || err){
				
				erreur = "Absence des lments obligatoires relatifs  la signature lectronique : Nom du signataire, Qualit du signataire, date de la signature, cumul des montants annuels (BLOQUANT)";
				
				String msg = getDocumentModel().getControlById(CTRL_ID).getMessage().getHtmlContent();
				msg = msg.replaceAll(MSG_ERREUR, erreur);
				message = "Signature globale";
				
				
				String path="@added:primary-key='"+bord_Exer+"-"+bord_TypBord+"-"+bord_IdBord+"'";
				String anoId = IdGenerator.nextId();
				String libelleLien = "Bordereau "+bord_IdBord+" Mandat "+piece_IdPce;
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Bordereau");
				xslParams.put("mandatId", piece_IdPce);
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
                                        getDocumentModel().getControlById(CTRL_ID).getId(),
                                        getDocumentModel().getControlById(CTRL_ID).getLibelle(),
                                        getDocumentModel().getDocumentId(),
                                        xpath.split("/")[1],
                                        (depense?BORDEREAU_DEPENSE:BORDEREAU_RECETTE), // lment importable
                                        bord_IdBord, // id lment importable
                                        localName,
                                        libelleLien,
                                        message + " - " + msg,
                                        getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
                                        getDocumentModel().getControlById(CTRL_ID).getSeverity(),
                                        path,
                                        xslParams);
				ano.addNode(new Node(bord_nodeId));
				ano1.add(ano);
				
			}
			
			if (sign_adresse.equals("") || sign_adresse.equals(" ") || sign_adresse.equals("0")){
				
				erreur = "Absence de l'lment relatif suivant : Lieu de la signature lectronique (NON BLOQUANT)";
				
				String msg = getDocumentModel().getControlById(CTRL_ID).getMessage().getHtmlContent();
				msg = msg.replaceAll(MSG_ERREUR, erreur);
				message = "Signature globale";
				
				
				String path="@added:primary-key='"+bord_Exer+"-"+bord_TypBord+"-"+bord_IdBord+"'";
				String anoId = IdGenerator.nextId();
				String libelleLien = "Bordereau "+bord_IdBord+" Mandat "+piece_IdPce;
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Bordereau");
				xslParams.put("mandatId", piece_IdPce);
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
                                        getDocumentModel().getControlById(CTRL_ID).getId(),
                                        getDocumentModel().getControlById(CTRL_ID).getLibelle(),
                                        getDocumentModel().getDocumentId(),
                                        xpath.split("/")[1],
                                        (depense?BORDEREAU_DEPENSE:BORDEREAU_RECETTE), // lment importable
                                        bord_IdBord, // id lment importable
                                        localName,
                                        libelleLien,
                                        message + " - " + msg,
                                        getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
                                        "NON BLOQUANT",
                                        path,
                                        xslParams);
				ano.addNode(new Node(bord_nodeId));
				ano1.add(ano);
			}
			return ano1;
			
		} else {
			return null;
		}
	}

	public void endDocument() throws SAXException {}
	
	public void characters(char[] ch, int start, int length, String xpath) {
		if (sign_nom_isPresent)
			nom.append(new String(ch,start,length));
		if (sign_qualite_isPresent)
			qualite.append(new String(ch,start,length));
		if (sign_date_isPresent)
			date.append(new String(ch,start,length));
		if (sign_adresse_isPresent)
			adresse.append(new String(ch,start,length));
	}
	
	public void ignorableWhitespace(char[] ch, int start, int length,String xpath) {}
	
	public void startPrefixMapping(String prefix, String uri, String xpath) {}
	
	public void endPrefixMapping(String prefix, String xpath) {}
	
	public void processingInstruction(String target, String data, String xpath) {}
	
	public void skippedEntity(String name, String xpath) {}
	
	public void setParameters(Hashtable<String,Object> params) {
		this.hParams=params;
	}
	
	public void setDocumentLocator(Locator locator) {}
}
